# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging

import six
from django import forms
from django.forms import CheckboxSelectMultiple
from django.utils.translation import gettext_lazy as _

from .validators import (OPTIONS, Regex, RegexValidator,
                         compress, decompress, flags_to_value)

logger = logging.getLogger(__name__)


class RegexFormField(forms.CharField):
    # widget = forms.Textarea

    def __init__(self, *args, **kwargs):
        super(RegexFormField, self).__init__(*args, **kwargs)
        self.validators.append(RegexValidator())

    def prepare_value(self, value):
        if value is None:
            return None
        if isinstance(value, six.string_types):
            return value

        return value.pattern

    def to_python(self, value):
        """Return a string."""
        return value


class RegexTextField(forms.Textarea):
    pass


class RegexFlagsWidget(forms.MultiWidget):
    template_name = 'django_regex/widgets/regex.html'
    widget = forms.Textarea

    def __init__(self, widgets, attrs=None):
        super(RegexFlagsWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        pattern, flags = decompress(value)
        return pattern, flags_to_value(flags)


class FlagsInput(CheckboxSelectMultiple):
    template_name = 'django_regex/widgets/flags.html'


class FlagsField(forms.MultipleChoiceField):
    pass


class RegexFlagsFormField(forms.MultiValueField):
    """
    Form field that validates credit card expiry dates.
    """
    default_error_messages = {
        'invalid': _('Enter a list of values.'),
        'incomplete': _('Enter a complete value.'),
        'invalid_regex': ''}

    def __init__(self, *args, **kwargs):
        error_messages = self.default_error_messages.copy()
        kwargs['require_all_fields'] = False
        self.flags_separator = kwargs.pop('flags_separator', None)
        # self.regex_widget = kwargs.pop('widget', forms.Textarea)
        # w = kwargs.get('widget', RegexFormField)
        if 'error_messages' in kwargs:  # pragma: no cover
            error_messages.update(kwargs['error_messages'])
        fields = (
            RegexFormField(widget=kwargs.get('widget', forms.Textarea),
                           error_messages={'invalid': error_messages['invalid_regex']}),
            FlagsField(required=False,
                       choices=OPTIONS,
                       widget=FlagsInput)
        )
        super(RegexFlagsFormField, self).__init__(fields, *args, **kwargs)
        self.widget = RegexFlagsWidget(widgets=[fields[0].widget,
                                                fields[1].widget,
                                                ])
        self.validators.append(RegexValidator())

    def compress(self, data_list):
        return compress(data_list, self.flags_separator)

    def prepare_value(self, value):
        out = value
        if isinstance(value, Regex):
            out = compress([value.pattern, value.flags], self.flags_separator)
        if isinstance(value, list):
            out = compress(value, self.flags_separator)
        return out



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging
import re

import six
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _

logger = logging.getLogger(__name__)

FLAGS = {
    'I': re.IGNORECASE,
    'M': re.MULTILINE,
    'L': re.LOCALE,
    'S': re.DOTALL,
    'U': re.UNICODE,
    'V': re.VERBOSE,
}

OPTIONS = (
    (int(re.I), _('ignore case (IGNORECASE)')),
    (int(re.M), _('make anchors look for newline (MULTILINE)')),
    (int(re.M), _('assume current 8-bit locale (LOCALE)')),
    (int(re.S), _('make dot match newline (DOTALL)')),
    (int(re.U), 'assume unicode locale (UNICODE)'),
    (int(re.X), _('ignore whitespace and comments (VERBOSE)')),
)

Regex = type(re.compile(''))

DEFAULT_SEPARATOR = getattr(settings, 'DJANGO_REGEX_SEPARATOR', chr(1))


def compress(data_list, separator=None):
    sep = separator or DEFAULT_SEPARATOR
    if data_list in [None, [], ()]:
        return sep.join(['', '0'])
    pattern, flags = data_list
    return sep.join([pattern, str(value_to_flags(flags))])


def decompress(value, separator=None):
    sep = separator or DEFAULT_SEPARATOR
    if value is None:
        return [None, 0]
    if isinstance(value, Regex):
        return value.pattern, value_to_flags(value.flags)

    try:
        pattern, flags = value.split(sep)
    except ValueError:
        return value, 0
    return pattern, value_to_flags(flags)


def flags_to_value(value):
    out = []
    int_value = int(value)
    if int_value & re.IGNORECASE:  # SRE_FLAG_IGNORECASE = 2
        out.append(int(re.IGNORECASE))
    if int_value & re.MULTILINE:  # SRE_FLAG_MULTILINE = 8
        out.append(int(re.MULTILINE))
    if int_value & re.DOTALL:  # SRE_FLAG_MULTILINE = 16
        out.append(int(re.DOTALL))
    if int_value & re.UNICODE:  # SRE_FLAG_UNICODE = 32
        out.append(int(re.UNICODE))
    if int_value & re.VERBOSE:  # SRE_FLAG_VERBOSE = 64
        out.append(int(re.VERBOSE))
    return out


def value_to_flags(value):
    """

    :param value: int, string, list ->
    :return:
    >>> value_to_flags(0) == 0
    >>> value_to_flags("0") == 0
    >>> value_to_flags("i") == 2
    >>> value_to_flags([2]) == 2
    >>> value_to_flags(['I']) == 2

    """
    if not value:
        return 0
    try:
        return int(value)
    except (ValueError, TypeError):
        pass

    if isinstance(value, six.string_types):
        value = [FLAGS[x.upper()] for x in value if x.upper() in FLAGS]

    return sum(map(int, value))


@deconstructible
class RegexValidator(object):
    message = _("`%(pattern)s` is not a valid regular expression")
    code = 'regex'

    def __call__(self, value):
        try:
            re.compile(value)
        except Exception:
            raise ValidationError(self.message,
                                  code=self.code,
                                  params={'pattern': value})



import pytest
from django.urls import reverse


@pytest.mark.django_db
def test_checker_success(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_test_regex', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = '[0-9]*'
    res.form['regex_1'] = [False, ]
    res.form['text'] = '123'
    res = res.form.submit()
    assert res.status_code == 200
    assert "SRE_Match" in str(res.content)


@pytest.mark.django_db
def test_checker_error(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_test_regex', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = '*'
    res.form['regex_1'] = [False, ]
    res.form['text'] = '123'
    res = res.form.submit()
    assert res.status_code == 200
    assert not "SRE_Match" in str(res.content)


@pytest.mark.django_db
def test_checker_fail(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_test_regex', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = '[a-z]$'
    res.form['regex_1'] = [False, False, False, False, False, False, ]
    res.form['text'] = '1'
    res = res.form.submit()
    assert res.status_code == 200
    assert not "SRE_Match" in str(res.content)


@pytest.mark.django_db
def test_checker_save(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_test_regex', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = '[a-z]*'
    res.form['regex_1'] = [False, ]
    res.form['text'] = '123'
    res = res.form.submit('save')
    assert res.status_code == 302



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import sys

import pytest

from django_regex.exceptions import InvalidPattern
from django_regex.utils import RegexList

pytestmark = pytest.mark.django_db


def test_init():
    rules = RegexList(['^abc$'])
    assert 'abc' in rules
    assert 'a' not in rules


def test_number():
    rules = RegexList()
    rules.append('\d*')
    assert 1 in rules
    assert 'a' not in rules


def test_append():
    rules = RegexList()
    rules.append('^abc$')
    assert 'abc' in rules
    assert 'a' not in rules


def test_setitem():
    rules = RegexList([''])
    rules[0] = '^abc$'
    assert 'abc' in rules
    assert 'a' not in rules


def test_error():
    with pytest.raises(InvalidPattern):
        RegexList(['*'])

    with pytest.raises(InvalidPattern):
        rules = RegexList()
        rules.append('**')


def test_repr():
    rules = RegexList(['.*', '[0-9]*'])
    if sys.version_info[0] < 3:
        assert str(rules) == "[u'.*', u'[0-9]*']"
    elif sys.version_info[0] == 3:
        assert str(rules) == "['.*', '[0-9]*']"



import os

HERE = os.path.abspath(os.path.dirname(__file__))
BASE_DIR = os.path.join(HERE, os.pardir, os.pardir, os.pardir)
SRC_DIR = os.path.join(BASE_DIR, 'src')
PACKAGE_DIR = os.path.join(SRC_DIR, 'django_regex')

DEBUG = True
STATIC_URL = '/static/'

SITE_ID = 1
ROOT_URLCONF = 'demo.urls'
SECRET_KEY = 'abc'
STATIC_ROOT = 'static'
MEDIA_ROOT = 'media'

INSTALLED_APPS = ['django.contrib.auth',
                  'django.contrib.contenttypes',
                  'django.contrib.sessions',
                  'django.contrib.sites',
                  'django.contrib.messages',
                  'django.contrib.staticfiles',
                  'django.contrib.admin',
    'admin_extra_urls',
                  'django_regex',
                  'demo',
                  ]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware']

LOCALE_PATHS = (
    os.path.join(PACKAGE_DIR, "locale"),
)

# TEMPLATE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        # 'DIRS': [
        #     str(APPS_DIR.path('templates')),
        # ],
        'OPTIONS': {
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            'debug': DEBUG,
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ],
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
                # Your stuff: custom template context processors go here
            ],
        },
    },
]

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'debug': {
            'format': '%(levelno)s:%(levelname)-8s %(name)s %(funcName)s:%(lineno)s:: %(message)s'
        }
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'debug'
        }
    },
    'loggers': {
        'django_regex': {
            'handlers': ['null'],
            'propagate': False,
            'level': 'DEBUG'
        }
    }
}

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': "db.sqlite",
    }
}



import os
import re
import sys

import pytest


def pytest_configure(config):
    here = os.path.dirname(__file__)
    sys.path.insert(0, here)


@pytest.fixture(scope='session')
def client(request):
    import django_webtest

    wtm = django_webtest.WebTestMixin()
    wtm.csrf_checks = False
    wtm._patch_settings()
    request.addfinalizer(wtm._unpatch_settings)
    app = django_webtest.DjangoTestApp()
    return app


@pytest.fixture
def demomodel(db):
    from demo.factories import DemoModelFactory
    return DemoModelFactory(regex='.*', name='name')


@pytest.fixture
def demomodel2(db):
    from demo.factories import DemoModel2Factory
    return DemoModel2Factory(regex=re.compile('^$', re.I + re.M), name='name')



from django.db import migrations, models

import django_regex.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DemoModel1',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('regex', django_regex.fields.RegexField()),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.CreateModel(
            name='DemoModel2',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('regex', django_regex.fields.RegexFlagsField(blank=True, null=True)),
            ],
            options={
                'ordering': ('id',),
            },
        ),
    ]



#!/usr/bin/env python
import os
import sys

here = os.path.abspath(os.path.join(os.path.dirname(__file__)))
rel = lambda *args: os.path.join(here, *args)

sys.path.insert(0, rel(os.pardir))


if __name__ == "__main__":
    sys.path.insert(0, 'src')
    sys.path.insert(0, 'tests/demoapp')
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "demo.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging
import re

from django import forms
from django.contrib import messages
from django.template.response import TemplateResponse

from admin_extra_urls.extras import ExtraUrlMixin, action
from django_regex.forms import RegexFlagsFormField
from django_regex.validators import decompress, compress

logger = logging.getLogger(__name__)


class TestRegexForm(forms.Form):
    regex = RegexFlagsFormField(
        # widget=forms.TextInput,
        required=False)
    text = forms.CharField(widget=forms.Textarea,
                           required=False)


class AdminRegexHelperMixin(ExtraUrlMixin):

    @action(label='Test Regex')
    def test_regex(self, request, id):
        target = self.get_object(request, id)
        text = request.GET.get('text', '')
        context = {'target': target,
                   'opts': self.model._meta,
                   'object': target,
                   }
        if request.method == 'POST':
            form = TestRegexForm(request.POST)
            if form.is_valid():
                value = form.cleaned_data['regex']
                pattern, options = decompress(value)
                try:
                    regex = re.compile(pattern, options)
                    m = regex.match(form.cleaned_data['text'])
                    if m:
                        context['match'] = m
                        context['groups'] = m.groups()
                        context['span'] = m.span()
                        context['groupdict'] = m.groupdict()
                    else:
                        self.message_user(request, "Does not match", messages.WARNING)

                    if 'save' in request.POST:
                        target.regex = regex
                        target.save()
                        self.message_user(request, "Regex updated")
                        return None

                except Exception as e:  # pragma: no cover
                    self.message_user(request, str(e), messages.ERROR)
        else:
            form = TestRegexForm(initial={'regex': target.regex.pattern,
                                          'text': text})
        context['form'] = form
        return TemplateResponse(request, 'admin/django_regex/regex.html', context)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging
import re

import pytest
import six
from django.urls import reverse

logger = logging.getLogger(__name__)


@pytest.mark.django_db
def test_admin_change(django_app, admin_user, demomodel):
    change_url = reverse('admin:demo_demomodel1_change', args=[demomodel.pk])
    res = django_app.get(change_url, user=admin_user)
    assert res.status_code == 200
    res.form['regex'] = 'xxx\d+'
    res = res.form.submit()
    assert res.status_code == 302
    demomodel.refresh_from_db()
    assert demomodel.regex.match('xxx22')


@pytest.mark.django_db
def test_admin_change_flags(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_change', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = 'xxx[0-9]+'
    res.form['regex_1'] = [False, False, False, False, False]
    res = res.form.submit()
    assert res.status_code == 302
    demomodel2.refresh_from_db()
    assert demomodel2.regex.match('xxx22')
    assert not demomodel2.regex.match('xxx')
    if six.PY3:
        assert demomodel2.regex.flags == re.UNICODE
    else:
        assert demomodel2.regex.flags == 0


@pytest.mark.django_db
def test_admin_change_flags_none(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_change', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = None
    res.form['regex_1'] = [True, True, True, True]
    res = res.form.submit()
    assert res.status_code == 302
    demomodel2.refresh_from_db()
    assert demomodel2.regex is None


@pytest.mark.django_db
def test_admin_change_flags_empty(django_app, admin_user, demomodel2):
    change_url = reverse('admin:demo_demomodel2_change', args=[demomodel2.pk])
    res = django_app.get(change_url, user=admin_user)
    res.form['regex_0'] = ''
    res.form['regex_1'] = [True, True, True, True]
    res = res.form.submit()
    assert res.status_code == 302
    demomodel2.refresh_from_db()
    assert demomodel2.regex is None



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import json
import logging

from django.core import serializers

from demo.models import DemoModel1

logger = logging.getLogger(__name__)


def test_serializer(demomodel):
    actual = serializers.serialize('json', DemoModel1.objects.all())
    data = json.loads(actual)
    assert data[0]["fields"]['regex'] == demomodel.regex.pattern



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.contrib.admin import ModelAdmin, register

from django_regex.admin_helper import AdminRegexHelperMixin
from .models import DemoModel1, DemoModel2


@register(DemoModel1)
class DemoAdmin1(ModelAdmin):
    list_display = ('name', 'regex')


@register(DemoModel2)
class DemoAdmin2(AdminRegexHelperMixin, ModelAdmin):
    list_display = ('name', 'regex', 'pattern', 'flags')

    def pattern(self, obj):
        if obj.regex:
            return obj.regex.pattern

    def flags(self, obj):
        if obj.regex:
            return obj.regex.flags



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import re

import pytest

from demo.models import DemoModel2
from django_regex.fields import RegexField, RegexFlagsField
from django_regex.forms import RegexFlagsFormField
from django_regex.validators import flags_to_value, value_to_flags, compress

pytestmark = pytest.mark.django_db


@pytest.mark.parametrize("value", [(re.I, re.M, re.DOTALL, re.U, re.VERBOSE),
                                   [2, 8, 16, 32, 64],
                                   'imsuv'
                                   ]
                         )
def test_value_to_flags1(value):
    assert value_to_flags(value) == 122


@pytest.mark.parametrize("value", [None, '', 0, '0'])
def test_value_to_flags2(value):
    assert value_to_flags(value) == 0


@pytest.mark.parametrize("value", [122, '122'])
def test_flags_to_value(value):
    assert flags_to_value(value) == [2, 8, 16, 32, 64]


def test_flags():
    f = RegexField(flags_separator='/')
    regex = f.clean('abc/2', None)
    assert regex.match('abc')
    assert regex.match('ABC')


def test_flags_humanized():
    f = RegexField(flags_separator='/')
    regex = f.clean('abc/i', None)
    assert regex.match('abc')
    assert regex.match('ABC')

    o = DemoModel2.objects.create(regex=compress(['aa', 2]))
    o.regex.match('AA')

    o = DemoModel2.objects.create(regex=compress(['aa', 'i']))
    o.regex.match('AA')


def test_flagsfield():
    f = RegexFlagsField()
    assert isinstance(f.formfield(), RegexFlagsFormField)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import logging

from django.db import models

from django_regex.fields import RegexField, RegexFlagsField

logger = logging.getLogger(__name__)


class DemoModel1(models.Model):
    name = models.CharField(max_length=100)
    regex = RegexField()

    class Meta:
        ordering = ("id",)
        app_label = 'demo'


class DemoModel2(models.Model):
    name = models.CharField(max_length=100)
    regex = RegexFlagsField(null=True, blank=True)

    class Meta:
        ordering = ("id",)
        app_label = 'demo'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import re

import six
from django.forms import modelform_factory

from demo.models import DemoModel2
from django_regex.forms import RegexFlagsFormField


def test_validate_fail():
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name', 'regex_0': '*', 'regex_1': ''}
    form = Form(data=data)
    assert not form.is_valid(), form.errors


def test_valid():
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name', 'regex_0': '.*', 'regex_1': ''}
    form = Form(data=data)
    assert form.is_valid()


# def test_required():
#     Form = modelform_factory(DemoModel2, exclude=())
#     data = {'name': 'name', 'regex_0': '', 'regex_1': ''}
#     form = Form(data=data)
#     assert not form.is_valid()


def test_none():
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name'}
    form = Form(data=data)
    assert form.is_valid()


def test_save(db):
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name', 'regex_0': 'aaa', 'regex_1': [int(re.I)]}
    form = Form(data=data)
    form.is_valid()
    obj = form.save()
    assert obj.regex.match('AAA')


def test_render_field(db, demomodel2):
    Form = modelform_factory(DemoModel2, fields=['regex'])
    form = Form(instance=demomodel2)
    rendered = form.as_p()
    m = re.findall(' checked ', rendered)
    if six.PY3:
        assert len(m) == 4, rendered
    else:
        assert len(m) == 3, rendered
    assert 'name="regex_0" value="^$"' in rendered, rendered


def test_render(db, demomodel):
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name', 'regex_0': '', 'regex_1': ''}
    form = Form(data=data)
    assert form.as_p()

    form = Form(instance=demomodel)
    assert form.as_p()

    demomodel.regex = None
    form = Form(instance=demomodel)
    assert form.as_p()


def test_instance(db, demomodel):
    Form = modelform_factory(DemoModel2, exclude=())
    form = Form({'name': demomodel.name,
                 'regex_0': demomodel.regex.pattern,
                 'regex_1': [],
                 }, instance=demomodel)
    assert form.initial['regex'] == demomodel.regex
    assert isinstance(form.fields['regex'], RegexFlagsFormField)

    assert form.is_valid(), form.errors

    obj = form.save()
    assert obj.regex == demomodel.regex



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging
import re

from django_regex.exceptions import InvalidPattern

logger = logging.getLogger(__name__)


class RegexList(list):
    def __init__(self, seq=None):
        regexx = []
        if seq:
            for el in seq:
                regexx.append(self._compile(el))
        super(RegexList, self).__init__(regexx)

    def __repr__(self):
        return str([r.pattern for r in self])

    def _compile(self, pattern, index=None):
        try:
            return re.compile(pattern)
        except (TypeError, re.error):
            raise InvalidPattern(pattern)

    def __setitem__(self, i, pattern):
        rex = self._compile(pattern)
        super(RegexList, self).__setitem__(i, rex)

    def append(self, pattern):
        rex = self._compile(pattern)
        super(RegexList, self).append(rex)

    def __contains__(self, target):
        t = str(target)
        for rex in self:
            m = rex.match(t)
            if m and m.group():
                return True
        return False




# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import re

import pytest
from django.core.exceptions import ValidationError
from django.db import connection

from django_regex.fields import RegexField
from django_regex.forms import RegexFormField, compress

pytestmark = pytest.mark.django_db


def test_type():
    f = RegexField()
    assert f.db_parameters(connection)['type'] == 'text'


def test_formfield():
    f = RegexField()
    assert isinstance(f.formfield(), RegexFormField)


def test_validation():
    f = RegexField()
    with pytest.raises(ValidationError):
        f.clean(None, None)

    with pytest.raises(ValidationError):
        f.clean('*', None)
    assert f.clean('.*', None)


def test_flags():
    f = RegexField()
    regex = f.clean('abc', None)
    assert regex.match('abc')
    assert not regex.match('ABC')

    f = RegexField(flags=re.I)
    regex = f.clean('abc', None)
    assert regex.match('abc')
    assert regex.match('ABC')

    f = RegexField(flags=re.I)
    regex = f.clean(compress(['abc', '32']), None)
    assert regex.match('abc')
    assert not regex.match('ABC')

    f = RegexField()
    regex = f.clean(compress(['abc', '2']), None)
    assert regex.match('abc')
    assert regex.match('ABC')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from factory import DjangoModelFactory


class DemoModelFactory(DjangoModelFactory):
    class Meta:
        model = 'demo.DemoModel1'
        # django_get_or_create = ('id',)


class DemoModel2Factory(DjangoModelFactory):
    class Meta:
        model = 'demo.DemoModel2'
        # django_get_or_create = ('id',)



#!/usr/bin/env python
import codecs
import imp
import os

from setuptools import find_packages, setup

ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))
init = os.path.join(ROOT, 'src', 'django_regex', '__init__.py')
app = imp.load_source('django_regex', init)


def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts)).read()


tests_requires = read('src/requirements/testing.pip')
install_requires = read('src/requirements/install.pip')
dev_requires = read('src/requirements/develop.pip')

setup(
    name=app.NAME,
    version=app.VERSION,
    url='https://github.com/saxix/django-regex',
    author='Stefano Apostolico',
    author_email='s.apostolico@gmail.com',
    license="MIT",
    description='Fields and utilities to work with regular expression in Django',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    install_requires=install_requires,
    tests_require=tests_requires,
    extras_require={
        'dev': dev_requires,
        'tests': tests_requires,
        'extra': ['admin-extra-urls',]
    },
    platforms=['linux'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Developers'
    ]
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.forms import modelform_factory

from demo.models import DemoModel1, DemoModel2
from django_regex.forms import RegexFormField
from django_regex.fields import RegexField


def test_validate_fail():
    Form = modelform_factory(DemoModel1, exclude=())
    data = {'name': 'name', 'regex': '*'}
    form = Form(data=data)
    assert not form.is_valid()


def test_valid():
    Form = modelform_factory(DemoModel1, exclude=())
    data = {'name': 'name', 'regex': '.*'}
    form = Form(data=data)
    assert form.is_valid()


def test_required():
    Form = modelform_factory(DemoModel1, exclude=())
    data = {'name': 'name', 'regex': ''}
    form = Form(data=data)
    assert not form.is_valid()


def test_none():
    Form = modelform_factory(DemoModel2, exclude=())
    data = {'name': 'name'}
    form = Form(data=data)
    assert form.is_valid()


def test_save(db):
    Form = modelform_factory(DemoModel1, exclude=())
    data = {'name': 'name', 'regex': 'aaa'}
    form = Form(data=data)
    form.is_valid()
    obj = form.save()
    assert obj.regex.match('aaa')


def test_render_field(db, demomodel):
    Form = modelform_factory(DemoModel1, fields=['regex'],
                             field_classes={RegexField: RegexFormField})
    form = Form(instance=demomodel)
    rendered = form.as_p()
    assert 'value=".*"' in rendered
    assert 'required' in rendered


def test_render(db, demomodel):
    Form = modelform_factory(DemoModel1, exclude=())
    data = {'name': 'name', 'regex': 'aaa'}
    form = Form(data=data)
    assert form.as_p()

    form = Form(instance=demomodel)
    assert form.as_p()

    demomodel.regex = None
    form = Form(instance=demomodel)
    assert form.as_p()


def test_instance(db, demomodel):
    Form = modelform_factory(DemoModel1, exclude=())
    form = Form({'name': demomodel.name,
                 'regex': demomodel.regex.pattern}, instance=demomodel)
    assert form.initial['regex'] == demomodel.regex
    assert isinstance(form.fields['regex'], RegexFormField)
    assert form.is_valid(), form.errors
    obj = form.save()
    assert obj.regex == demomodel.regex


####
####

def aaaa():
    field = RegexFormField()
    assert field.clean('aaa')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from django import template

register = template.Library()
register_tag = register.assignment_tag if hasattr(register, 'assignment_tag') else register.simple_tag


@register_tag
def inspect(match, obj=None, target=None):
    """Converts a string into all lowercase"""
    return {'groups': match.groups,
            }





import re
from sphinx import addnodes, roles
from sphinx.util.console import bold
from sphinx.util.compat import Directive
try:
    from sphinx.writers.html import SmartyPantsHTMLTranslator as HTMLTranslator
except ImportError:  # Sphinx 1.6+
    from sphinx.writers.html import HTMLTranslator

# RE for option descriptions without a '--' prefix
simple_option_desc_re = re.compile(
    r'([-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)')


def setup(app):
    app.add_crossref_type(
        directivename="setting",
        rolename="setting",
        indextemplate="pair: %s; setting",
    )
    app.add_crossref_type(
        directivename="templatetag",
        rolename="ttag",
        indextemplate="pair: %s; template tag"
    )
    app.add_crossref_type(
        directivename="templatefilter",
        rolename="tfilter",
        indextemplate="pair: %s; template filter"
    )
    app.add_crossref_type(
        directivename="fieldlookup",
        rolename="lookup",
        indextemplate="pair: %s; field lookup type",
    )
    app.add_config_value('next_version', '0.0', True)
    app.add_directive('versionadded', VersionDirective)
    app.add_directive('versionchanged', VersionDirective)
    app.add_crossref_type(
        directivename="release",
        rolename="release",
        indextemplate="pair: %s; release",
    )

class DjangoHTMLTranslator(HTMLTranslator):
    """
    Django-specific reST to HTML tweaks.
    """

    # Don't use border=1, which docutils does by default.
    def visit_table(self, node):
        self.context.append(self.compact_p)
        self.compact_p = True
        self._table_row_index = 0  # Needed by Sphinx
        self.body.append(self.starttag(node, 'table', CLASS='docutils'))

    def depart_table(self, node):
        self.compact_p = self.context.pop()
        self.body.append('</table>\n')

    def visit_desc_parameterlist(self, node):
        self.body.append('(')  # by default sphinx puts <big> around the "("
        self.first_param = 1
        self.optional_param_level = 0
        self.param_separator = node.child_text_separator
        self.required_params_left = sum([isinstance(c, addnodes.desc_parameter)
                                         for c in node.children])

    def depart_desc_parameterlist(self, node):
        self.body.append(')')


    version_text = {
        'deprecated': 'Deprecated in Django Regex %s',
        'versionchanged': 'Changed in Django Regex %s',
        'versionadded': 'New in Django Regex %s',
    }

    def visit_versionmodified(self, node):
        self.body.append(
            self.starttag(node, 'div', CLASS=node['type'])
        )
        version_text = self.version_text.get(node['type'])
        if version_text:
            title = "%s%s" % (
                version_text % node['version'],
                ":" if len(node) else "."
            )
            self.body.append('<span class="title">%s</span> ' % title)

    def depart_versionmodified(self, node):
        self.body.append("</div>\n")

    # Give each section a unique ID -- nice for custom CSS hooks
    def visit_section(self, node):
        old_ids = node.get('ids', [])
        node['ids'] = ['s-' + i for i in old_ids]
        node['ids'].extend(old_ids)
        SmartyPantsHTMLTranslator.visit_section(self, node)
        node['ids'] = old_ids



class VersionDirective(Directive):
    has_content = True
    required_arguments = 1
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        if len(self.arguments) > 1:
            msg = """Only one argument accepted for directive '{directive_name}::'.
            Comments should be provided as content,
            not as an extra argument.""".format(directive_name=self.name)
            raise self.error(msg)

        env = self.state.document.settings.env
        ret = []
        node = addnodes.versionmodified()
        ret.append(node)

        if self.arguments[0] == env.config.next_version:
            node['version'] = "Development version"
        else:
            node['version'] = self.arguments[0]

        node['type'] = self.name
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        env.note_versionchange(node['type'], node['version'], node, self.lineno)
        return ret



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import pytest

from demo.models import DemoModel1, DemoModel2

pytestmark = pytest.mark.django_db


def test_save():
    m = DemoModel1()
    m.regex = '^aaa$'
    m.save()
    assert m.pk
    assert m.regex.match('aaa')


def test_none():
    assert DemoModel2.regex is None


def test_save_none():
    m = DemoModel2()
    m.save()
    assert m.pk
    with pytest.raises(AttributeError):
        assert m.regex.match('aaa')


def test_lookup_equal():
    m = DemoModel1()
    m.regex = '^aaa$'
    m.save()
    assert DemoModel1.objects.filter(regex='^aaa$').exists()


def test_lookup_startswith():
    m = DemoModel1()
    m.regex = '^aaa$'
    m.save()
    assert DemoModel1.objects.filter(regex__startswith='^').exists()


def test_lookup_endswith():
    m = DemoModel1()
    m.regex = '^aaa$'
    m.save()
    assert DemoModel1.objects.filter(regex__endswith='$').exists()


def test_lookup_contains():
    m = DemoModel1()
    m.regex = '^aaa$'
    m.save()
    assert DemoModel1.objects.filter(regex__contains='aaa').exists()



import re

from django import forms
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.fields import NOT_PROVIDED
from django.utils.translation import ugettext_lazy as _
from .forms import RegexFlagsFormField, RegexFormField
from .validators import Regex, RegexValidator, compress, decompress

rex = re.compile('')


class RegexFieldDescriptor(object):
    def __init__(self, field):
        self.field = field

    def __get__(self, obj, type=None):
        if obj is None:
            return None
        return obj.__dict__.get(self.field.name)

    def __set__(self, obj, value):
        obj.__dict__[self.field.name] = self.field.to_python(value)


class RegexField(models.Field):
    descriptor = RegexFieldDescriptor
    form_class = RegexFormField
    widget = forms.TextInput

    def __init__(self, *args, **kwargs):
        self.flags_separator = kwargs.pop('flags_separator', None)
        self.flags = kwargs.pop('flags', 0)
        super(RegexField, self).__init__(*args, **kwargs)
        self.validators.append(RegexValidator)

    def contribute_to_class(self, cls, name, private_only=False, virtual_only=NOT_PROVIDED):
        self.set_attributes_from_name(name)
        self.model = cls
        cls._meta.add_field(self)
        setattr(cls, self.name, self.descriptor(self))

    def get_internal_type(self):
        return 'TextField'

    def to_python(self, value):
        if not value:
            return None
        pattern, flags = value, self.flags
        if isinstance(value, Regex):
            return value
        try:
            pattern, flags = decompress(value, self.flags_separator)
        except ValueError:
            pass
        if not pattern:
            return None
        try:
            return re.compile(pattern, flags)
        except Exception:
            raise ValidationError(_("`%(pattern)s` is not a valid regular expression"),
                                  params={'pattern': pattern})

    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname)
        if not value:  # pragma: no cover
            return None
        return value.pattern

    def value_to_string(self, obj):
        """
        Return a string value of this field from the passed obj.
        This is used by the serialization framework.
        """
        return self.value_from_object(obj).pattern

    def formfield(self, form_class=None, choices_form_class=None, **kwargs):
        kwargs['widget'] = self.widget
        return super(RegexField, self).formfield(self.form_class,
                                                 choices_form_class, **kwargs)


class RegexFlagsField(RegexField):
    form_class = RegexFlagsFormField

    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.attname)
        if not value:
            return None
        return compress([value.pattern, value.flags])
